CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                C
C     Monte Carlo Simulation of Regular Fixed Random Surafce     C
C                                                                C
C     version 0.1,  May 19, 1993.   LH                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  About this version :                                          C
C     1. regular fixed triangular mesh                           C
C     2. at this moment, only Gauss action term used             C
C     3. may need to use double precision for some variable      C
C        to take care rounding ....                              C
C     4. no optimization                                         C
C     5. double decision really slower ?                         C
C     ...... ?                                                   C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
    
      PROGRAM random_surface_simulation

      INCLUDE "reg.h"

      INTEGER i
      real*8 error1, error2
      
      CALL init
 
      DO i = 1, thermalization_sweeps
        CALL update_sites
      END DO

      DO i = 1, sweeps
        CALL update_sites
        CALL measure
        CALL average
      END DO

C     strictly should be divided by (sweeps - 1)
      error1 = sqrt((ave_gauss2 - ave_gauss**2)/sweeps)
      error2 = sqrt((ave_accept_rate2 - ave_accept_rate**2)/sweeps)

C     The Gauss value should be 1.5*(n-1) within error1.
      print *, "Gauss average of", n," sites is : ",
     $          ave_gauss, "  +/- ", error1
      print *, "Site update acceptance rate is :", 
     $         ave_accept_rate, "+/- ", error2

      STOP
      END


CCCCCCCCCCCCCCCCCCCCC       INITIALIZATION        CCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE init

      INCLUDE "reg.h"

      INTEGER idim,inx,iny
      INTEGER seed1, seed2
      REAL scale
      REAL ranmar

      print *, "Enter random seed1 ( between 0 and 31328 )"
      read *, seed1
      print *, "seed1 = ", seed1
      print *, "Enter random seed2 ( between 0 and 31328 )"
      read *, seed2
      print *, "seed2 = ", seed2
      print *, "Enter the number of sweeps for thermalization"
      read *, thermalization_sweeps
      print *, "thermalization_sweeps = ", thermalization_sweeps
      print *, "Enter the number of sweeps for update ?"
      read *, sweeps
      print *, "sweeps = ", sweeps

      print *, "Input is done, the program is running. Please wait ..."

      scale = 1.0

      ave_gauss = 0.0
      ave_gauss2 = 0.0
      ave_accept_rate = 0.0
      ave_accept_rate2 = 0.0

      CALL rmarin(seed1,seed2) ! initialize the random number

      DO iny = 1, ny
         DO inx = 1, nx
            DO idim = 1, dim
               field(idim, inx, iny) = scale * (.5-ranmar()) ! initial field
            END DO
         END DO
      END DO

C     Look-up table of neighbors to impose periodic boundary conditions.
      DO inx = 1, nx
         pos(inx) = MOD(inx,nx) + 1
         neg(inx) = MOD(inx+nx-2,nx) + 1
      END DO
     
      DO iny = 1, ny
         DO inx = 1, nx
            local_gauss(inx,iny) = 0.0
         END DO
      END DO

      RETURN
      END


C********************  update_sites  ************************************C
      SUBROUTINE update_sites

      INCLUDE "reg.h"
      
      INTEGER inx, iny, i
      REAL ranmar
      REAL*8 trial_local_gauss, trial_x, trial_y, trial_z, diffE
      REAL*8 old_local_gauss

      INTEGER accept_counter

      accept_counter = 0

      do i = 1, n
         
         inx = nx * ranmar() + 1 
         iny = nx * ranmar() + 1

            trial_x = field(1,inx,iny) + trial_scale * (ranmar() - 0.5)
            trial_y = field(2,inx,iny) + trial_scale * (ranmar() - 0.5)
            trial_z = field(3,inx,iny) + trial_scale * (ranmar() - 0.5)

            trial_local_gauss = 
     &           (trial_x - field(1,pos(inx),iny)) ** 2 + 
     &           (trial_y - field(2,pos(inx),iny)) ** 2 + 
     &           (trial_z - field(3,pos(inx),iny)) ** 2 + 
     &           (trial_x - field(1,neg(inx),iny)) ** 2 + 
     &           (trial_y - field(2,neg(inx),iny)) ** 2 + 
     &           (trial_z - field(3,neg(inx),iny)) ** 2 + 
     &           (trial_x - field(1,inx,pos(iny))) ** 2 + 
     &           (trial_y - field(2,inx,pos(iny))) ** 2 + 
     &           (trial_z - field(3,inx,pos(iny))) ** 2 + 
     &           (trial_x - field(1,inx,neg(iny))) ** 2 + 
     &           (trial_y - field(2,inx,neg(iny))) ** 2 + 
     &           (trial_z - field(3,inx,neg(iny))) ** 2 + 
     &           (trial_x - field(1,pos(inx),pos(iny))) ** 2 + 
     &           (trial_y - field(2,pos(inx),pos(iny))) ** 2 + 
     &           (trial_z - field(3,pos(inx),pos(iny))) ** 2 + 
     &           (trial_x - field(1,neg(inx),neg(iny))) ** 2 + 
     &           (trial_y - field(2,neg(inx),neg(iny))) ** 2 + 
     &           (trial_z - field(3,neg(inx),neg(iny))) ** 2  

            old_local_gauss = 
     &           (field(1,inx,iny) - field(1,pos(inx),iny)) ** 2 + 
     &           (field(2,inx,iny) - field(2,pos(inx),iny)) ** 2 + 
     &           (field(3,inx,iny) - field(3,pos(inx),iny)) ** 2 + 
     &           (field(1,inx,iny) - field(1,neg(inx),iny)) ** 2 + 
     &           (field(2,inx,iny) - field(2,neg(inx),iny)) ** 2 + 
     &           (field(3,inx,iny) - field(3,neg(inx),iny)) ** 2 + 
     &           (field(1,inx,iny) - field(1,inx,pos(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,inx,pos(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,inx,pos(iny))) ** 2 + 
     &           (field(1,inx,iny) - field(1,inx,neg(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,inx,neg(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,inx,neg(iny))) ** 2 + 
     &           (field(1,inx,iny) - field(1,pos(inx),pos(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,pos(inx),pos(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,pos(inx),pos(iny))) ** 2 + 
     &           (field(1,inx,iny) - field(1,neg(inx),neg(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,neg(inx),neg(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,neg(inx),neg(iny))) ** 2  


C     Metroplis procedure

            diffE =  trial_local_gauss - old_local_gauss
            IF (diffE .GE. 0) THEN
               IF (exp(-diffE) .LT. ranmar()) goto 22
            END IF
            field(1,inx,iny) = trial_x
            field(2,inx,iny) = trial_y
            field(3,inx,iny) = trial_z
            accept_counter = accept_counter + 1

 22         continue
      END DO

      accept_rate = float(accept_counter)/n

      RETURN
      END


C********************   measure   ***************************************C
      SUBROUTINE measure

      INCLUDE "reg.h"

      INTEGER inx, iny

      gauss = 0.0

      DO iny = 1, ny
         DO inx = 1, nx
            gauss = gauss + 
     &           (field(1,inx,iny) - field(1,pos(inx),iny)) ** 2 + 
     &           (field(2,inx,iny) - field(2,pos(inx),iny)) ** 2 + 
     &           (field(3,inx,iny) - field(3,pos(inx),iny)) ** 2 + 
     &           (field(1,inx,iny) - field(1,neg(inx),iny)) ** 2 + 
     &           (field(2,inx,iny) - field(2,neg(inx),iny)) ** 2 + 
     &           (field(3,inx,iny) - field(3,neg(inx),iny)) ** 2 + 
     &           (field(1,inx,iny) - field(1,inx,pos(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,inx,pos(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,inx,pos(iny))) ** 2 + 
     &           (field(1,inx,iny) - field(1,inx,neg(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,inx,neg(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,inx,neg(iny))) ** 2 + 
     &           (field(1,inx,iny) - field(1,pos(inx),pos(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,pos(inx),pos(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,pos(inx),pos(iny))) ** 2 + 
     &           (field(1,inx,iny) - field(1,neg(inx),neg(iny))) ** 2 + 
     &           (field(2,inx,iny) - field(2,neg(inx),neg(iny))) ** 2 + 
     &           (field(3,inx,iny) - field(3,neg(inx),neg(iny))) ** 2  
         END DO
      END DO

      gauss = 0.5 * gauss  ! because we have doubly summed each link

      RETURN
      END

C*******************  average  ******************************************C
      SUBROUTINE average

      INCLUDE "reg.h"
 
      ave_gauss = ave_gauss + gauss/sweeps
      ave_gauss2 = ave_gauss2 + gauss*gauss/sweeps
      ave_accept_rate = ave_accept_rate + accept_rate/sweeps
      ave_accept_rate2 = ave_accept_rate2 + accept_rate**2/sweeps

      RETURN
      END


C************************************************************************C
C     RANDOM NUMBER GENERATOR ROUTINES.                                  C
C************************************************************************C

      SUBROUTINE RMARIN(IJ,KL)
C This is the initialization routine for the random number generator
C     RANMAR()
C NOTE: The seed variables can have values between:    0 <= IJ <= 31328
C                                                      0 <= KL <= 30081
      REAL u(97), c, cd, cm
      INTEGER i97, j97, ivec
      LOGICAL test
      COMMON /raset1/ u, c, cd, cm, i97, j97, ivec, test

      IF( ij .LT. 0  .OR.  ij .GT. 31328  .OR.
     *    kl .LT. 0  .OR.  kl .GT. 30081 ) THEN
        PRINT '(A)', ' The first random number seed must have a value
     *between 0 and 31328'
        PRINT '(A)',' The second seed must have a value between 0 and
     *30081'
        STOP
      ENDIF
      i = mod(ij/177, 177) + 2
      j = mod(ij    , 177) + 2
      k = mod(kl/169, 178) + 1
      l = mod(kl,     169)
      DO 2 ii = 1, 97
        s = 0.0
        t = 0.5
        DO 3 jj = 1, 24
          m = mod(mod(i*j, 179)*k, 179)
          i = j
          j = k
          k = m
          l = mod(53*l+1, 169)
          IF (mod(l*m, 64) .ge. 32) then
            s = s + t
          ENDIF
          t = 0.5 * t
    3   CONTINUE
        u(ii) = s
    2 CONTINUE
      c = 362436.0 / 16777216.0
      cd = 7654321.0 / 16777216.0
      cm = 16777213.0 /16777216.0
      i97 = 97
      j97 = 33
      test = .true.
      RETURN
      END
C **************************************************************************
C **************************************************************************
      FUNCTION ranmar()
      IMPLICIT NONE
      REAL rand_u(97), rand_c, rand_cd, rand_cm, rand_uni, ranmar
      INTEGER rand_i97, rand_j97, rand_ivec
      LOGICAL rand_test
      COMMON /raset1/ rand_u, rand_c, rand_cd, rand_cm, rand_i97
     $     , rand_j97, rand_ivec, rand_test

      IF( .NOT. rand_test ) THEN
        PRINT '(A)',' Call the init routine (RMARIN) before calling RAN
     *MAR'
        STOP
      ENDIF
      rand_uni = rand_u(rand_i97) - rand_u(rand_j97)
      IF( rand_uni .LT. 0.0 ) rand_uni = rand_uni + 1.0
      rand_u(rand_i97) = rand_uni
      rand_i97 = rand_i97 - 1
      IF(rand_i97 .EQ. 0) rand_i97 = 97
      rand_j97 = rand_j97 - 1
      IF(rand_j97 .EQ. 0) rand_j97 = 97
      rand_c = rand_c - rand_cd
      IF( rand_c .LT. 0.0 ) rand_c = rand_c + rand_cm
      rand_uni = rand_uni - rand_c
      IF( rand_uni .LT. 0.0 ) rand_uni = rand_uni + 1.0
      ranmar = rand_uni
      RETURN
      END
C **************************************************************************

      BLOCK DATA
      REAL u(97), c, cd, cm
      INTEGER i97, j97, ivec
      LOGICAL test
      COMMON /raset1/ u, c, cd, cm, i97, j97, ivec, test
      DATA test /.false./
      END
