CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                  C
C     This program is Monte Carlo simulation of dynamically        C
C     triangularized random surfaces.                              C
C     Writen by Enzo Marinari, June 1992                           C
C     Optimized by Leping Han, Aug. 1992                           C
C     Syracuse University, Aug. 1992                               C
C                                                                  C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


      PROGRAM pippo

      INCLUDE 'string.h'
      
      CALL init

      IF(start_mode.ne.-1)CALL init_field

CC    Thermalize the mesh. 
      DO i_sweep=1, n_discard
        CALL update_many_sites
      ENDDO

CC    Update the mesh and measure the observables.
      DO i_sweep = 1, n_sweeps
        DO i_correlated = 1, n_correlated
          DO i_one_sweep =1, time_scale
            CALL update_many_sites
          ENDDO
          CALL measure
          CALL average_1
        ENDDO
        CALL average_2
      ENDDO

      CALL ciao

      STOP
  123 FORMAT(a1)
      END

C______________________________________________________________________

CC    Save the mesh in a file.
 
      SUBROUTINE ciao

      INCLUDE 'string.h'
      INTEGER id, is, im, io, i_two
      REAL * 8 diff

      PRINT *,'Flip acceptance rate _ ',tot_lacc
      PRINT *,'Site acceptance rate _ ',tot_sacc

      WRITE(f_out,*)'Flip acceptance rate _ ',tot_lacc
      WRITE(f_out,*)'Site acceptance rate _ ',tot_sacc
      
        DO i_two=1,2
          diff =  (tot_obs(i_two,2,3)-tot_obs(i_two,1,3)**2)
     $          * inv_n_sweeps_1
          IF(diff.GT.0)THEN
            tot_obs(i_two,3,3) = sqrt(diff)
          ELSE
            tot_obs(i_two,3,3) = -999999.
          ENDIF
        ENDDO

        DO i_two=1,2
          PRINT*,i_two,phrase(3),tot_obs(i_two,1,3),' +/- '
     $            ,tot_obs(i_two,3,3)
          WRITE(f_out,*)i_two,phrase(3),tot_obs(i_two,1,3),' +/- '
     $            ,tot_obs(i_two,3,3)
        ENDDO


      RETURN
      END

C_______________________________________________________________________

CC    Read input parameters and initialize the variables.
CC    If the start_mode flag is 0, set_up a mesh. If the start_mode
CC    flag is -1, read mesh from the saved mesh file from previous 
CC    run and continue the run.

      SUBROUTINE init

      INCLUDE 'string.h'

      INTEGER i, io, i_two, i_three, m0
      INTEGER id, is, im
      INTEGER rnx, rny, rn, ri_seed1, ri_seed2, rtime_scale, rn_sweeps
      INTEGER rn_correlated, rn_discard,rwrite_all
      REAL*4 a, rlambda, rtrial
      real ranmar

CC    please, keep the phrases 'one word' ('xysfsdf', no blanks)
      phrase( 1) = '|q-6|     '
      phrase( 2) = 'log(q)    '
      phrase( 3) = 'GaussS    '
      phrase( 4) = 'q_max     '
      phrase( 5) = 'Ra        '
      phrase( 6) = 'Se        '
      phrase( 7) = 'I(|K|)    '
      phrase( 8) = 'Area      '
      phrase( 9) = 'I(|K-R|)  '
      phrase(10) = 'I(R**2)   '
      phrase(11) = 'I(K**2)   '
      phrase(12) = 'I(K*R)    '

      PRINT*,' nx = ', nx, ' ny = ', ny, ' n = ',n
      PRINT *, 'Input first i_seed (0 <= seed <= 31328) _ '
      PRINT *, 'NOT USED IF START_MODE = -1'
      READ(5,*)i_seed1
      PRINT *,' i_seed1 = ', i_seed1
      PRINT *, 'Input second i_seed (0 <= seed <= 30081) _ '
      PRINT *, 'NOT USED IF START_MODE = -1'
      READ(5,*)i_seed2
      PRINT *,' i_seed2 = ', i_seed2
      PRINT *, 'Input time_scale (how many multiples of 3n per measure'
      READ(5,*)time_scale
      PRINT *,' time_scale = ', time_scale
      PRINT *, 'Input n_sweeps (independent measurements)_ '
      READ(5,*)n_sweeps
      PRINT *,' n_sweeps = ', n_sweeps
      PRINT *, 'Input n_correlated (internal loop)_ '
      READ(5,*)n_correlated
      PRINT *,' n_correlated = ', n_correlated
      PRINT *, 'Input n_discard (internal loop)_ '
      READ(5,*)n_discard
      PRINT *,' n_discard = ', n_discard
      PRINT *, 'Input lambda_ '
      READ(5,*)lambda
      PRINT *,' lambda = ', lambda
      PRINT *, 'Input trial_ '
      READ(5,*)trial
      PRINT *,' trial = ', trial
      PRINT *,'Input 1 if write all _'
      READ(5,*)write_all
      PRINT *,'write_all = ', write_all
      PRINT *,'Input start_mode (-1 for continuing from aaa.ico)'
      READ(5,*)start_mode
      PRINT *,'start_mode = ', start_mode

CC    calculate all inverse number to avoid spending too much time
CC    calculating division for the later use.
      inv_n = 1.0/float(n)
      inv_3n = 1.0/float(3*n)
      inv_n6 = 1.0/float(n6)
      inv_n_sweeps = 1.0/float(n_sweeps)
      inv_n_sweeps_1 = 1.0/float(n_sweeps-1)
      inv_n_correlated = 1.0/float(n_correlated)
      third_two_pi = two_pi/3.0

      CALL rmarin(i_seed1,i_seed2)

      scale = 1.0
      md_half = - float(dim) / 2.0
      anx2 = float(nx) / 2.0 - .5
      any2 = float(ny) / 2.0 - .5

      if(start_mode .ne. -1)then
         DO i=1,200
            a = ranmar()
         ENDDO
         CALL set_up_mesh
      endif

      ind_lacc  = 0.0
      ind_sacc  = 0.0
      DO io=1,n_obs
        DO i_two =1,2
          ind_obs(i_two,io)=0.0
          DO i_three=1,3
            tot_obs(i_two,i_three,io)=0.0
          ENDDO
        ENDDO
      ENDDO

      tot_lacc   = 0.0
      tot_sacc   = 0.0

      RETURN
      END

C_______________________________________________________________________

CC    This routine is called by routine init when start_mode = 0.
CC    It initializes the coordinates of nodes in three dimensional
CC    space.

      SUBROUTINE init_field

      INCLUDE 'string.h'

      INTEGER d0, s0, m0, s1
      INTEGER random_start, old_conf
      INTEGER flat_start_xy, flat_start_xz, flat_start_yz
      INTEGER bulk_start_xy, bulk_start_xz, bulk_start_yz

      REAL scratch
      real ranmar

      old_conf = -1
      random_start = 0

      IF(start_mode .EQ. random_start)THEN
        DO d0=0,dm
          DO s0=0,nm
            field(d0,s0) = scale * (.5-ranmar())
          ENDDO
        ENDDO
      ENDIF

      DO m0 = 0, n6m
        square_m(m0) = 0.0
        s0 = s_site_m(m0)
        s1 = s_neighbor_m(m0)
        DO d0 = 0, dm
          scratch = field(d0,s0)-field(d0,s1)
          square_m(m0) = square_m(m0) + scratch * scratch
        ENDDO
      ENDDO

      RETURN
  123 FORMAT(a1)
      END

C_______________________________________________________________________


CC    This routine is called by routine "init" when start_mode = 0.
CC    It does the works to set up the geometric configuration of 
CC    the mesh. Each node has six links. The geometry is a torus.

      SUBROUTINE set_up_mesh

      INCLUDE 'string.h'

      INTEGER site, site1, s0, s2, m_start, m_start2
      integer nc,nc1,i,j
      DO site = 0 , nm
        m_start_s(site) = 6 * site
        n_connectivity_s(site) = 6
      ENDDO

      DO site = 0 , nm
        s_neighbor_m(6*site+0) = site + 1
        s_neighbor_m(6*site+1) = site + nx + 1
        s_neighbor_m(6*site+2) = site + nx
        s_neighbor_m(6*site+3) = site - 1
        s_neighbor_m(6*site+4) = site - nx - 1
        s_neighbor_m(6*site+5) = site - nx

        DO site1 = 0, 5
          m_next_m  (6*site+site1) = 6*site + site1 + 1
          m_former_m(6*site+site1) = 6*site + site1 - 1
          s_site_m  (6*site+site1) = site
        ENDDO
        m_next_m   ( 6*site + 5 ) = 6*site
        m_former_m ( 6*site     ) = 6*site + 5

        IF(site .LT. nx)THEN
          s_neighbor_m(6*site+5) = site + nx*(ny-1)
          s_neighbor_m(6*site+4) = site + nx*(ny-1) - 1
        ENDIF
        IF(nx*ny-1-site .lt. nx)then
          s_neighbor_m(6*site+2) = site - nx*(ny-1)
          s_neighbor_m(6*site+1) = site - nx*(ny-1) + 1
        ENDIF
        IF(mod(site,nx) .eq. nx-1)then
          s_neighbor_m(6*site+0) = site - nx + 1
          s_neighbor_m(6*site+1) = site + 1
        ENDIF
        IF(mod(site,nx) .eq. 0)then
          s_neighbor_m(6*site+3) = site + nx - 1
          s_neighbor_m(6*site+4) = site - 1
        ENDIF
      ENDDO

      s_neighbor_m(0+4) = nx * ny - 1
      s_neighbor_m(6*(nx*ny-1)+1) = 0

      DO s0=0, nm
        m_start = m_start_s(s0)
        nc = n_connectivity_s(s0)
        do i=1,nc
           s2 = s_neighbor_m(m_start)
           m_start2 = m_start_s(s2)
           nc1 = n_connectivity_s(s2)
           do j=1,nc1
              IF(s0.EQ.s_neighbor_m(m_start2))GOTO 52
              m_start2 = m_next_m(m_start2)
           enddo
 52        CONTINUE

CC    here m_start and m_start2 are the two opposite links.
           m_back_m(m_start ) = m_start2
           m_back_m(m_start2) = m_start
           m_start = m_next_m(m_start)
        enddo
      ENDDO

      RETURN
      END
C_______________________________________________________________________

CC    Average the physical observables over the inner loop sweeps
CC    ( see main routine ).

      SUBROUTINE average_1

      INCLUDE 'string.h'
      INTEGER io

      ind_lacc   = ind_lacc  + sw_lacc * inv_n_correlated
      ind_sacc   = ind_sacc  + sw_sacc * inv_n_correlated

        ind_obs(1,3) = ind_obs(1,3)
     $        +sw_obs(3) * inv_n_correlated
        ind_obs(2,3) = ind_obs(2,3)
     $        +sw_obs(3)*sw_obs(3) * inv_n_correlated

      RETURN
      END

C_______________________________________________________________________

CC    Average the physical observables over the outer loop sweeps
CC    ( see main routine ).

      SUBROUTINE average_2

      INCLUDE 'string.h'
      INTEGER i, io

      PRINT *,i_sweep
      PRINT *,'lacc,sacc ',ind_lacc, ind_sacc

        PRINT *, phrase(3),ind_obs(1,3), ind_obs(2,3)


      tot_lacc  = tot_lacc + ind_lacc * inv_n_sweeps
      tot_sacc  = tot_sacc + ind_sacc * inv_n_sweeps

        DO i=1,2
          tot_obs(i,1,3) = tot_obs(i,1,3)
     $           + ind_obs(i,3) * inv_n_sweeps
          tot_obs(i,2,3) = tot_obs(i,2,3)
     $           + ind_obs(i,3)*ind_obs(i,3) * inv_n_sweeps
        ENDDO

      ind_lacc = 0.0
      ind_sacc = 0.0

      DO io=1,n_obs
        DO i=1,2
          ind_obs(i,io) = 0.0
        ENDDO
      ENDDO

      RETURN
      END

C_______________________________________________________________________

CC    Physical observables measures.
CC    Save the measures in data files.

      SUBROUTINE measure

      INCLUDE 'string.h'

      INTEGER io, s0, m0, m_start, nc, sn, sr,link
      REAL f0, f1, f2, qmax, arg_scratch, arg_scratch1
      REAL fn0, fn1, fn2, fr0, fr1, fr2
      REAL site_d_angle, scalar_p, square_r, sq1, sq2
      REAL area, site_d_area
      REAL bari(0:dm), scratch(0:dm)
      REAL tmp
      INTEGER i,tmp_link

CC    unroll DO loop here gives a little improvement
      DO io=1, n_obs
        sw_obs(io) = 0.0
      ENDDO

      DO m0 = 0, n6m
        sw_obs(3) = sw_obs(3) + square_m(m0)
      ENDDO

      sw_obs(3) = 0.5 * sw_obs(3)

      RETURN
  123 FORMAT(a1)
  181 FORMAT(1x,6g13.6)
      END

C______________________________________________________________________

C_______________________________________________________________________

CC    Try to flip all NODES based on Metropolis procedures.

      SUBROUTINE update_many_sites

      INCLUDE 'string.h'

      INTEGER i_flip, m_start, s0, s1, s3, s4
      INTEGER m_forgot, i, icount, m_start1, m_start2, mem
      integer nc,r_link,l_link

      REAL ft(0:dm)
      REAL r_s1(0:dm),r_s3(0:dm)
      REAL norm_dt, dott_31, dott_41
      REAL factort31, factort41
      REAL h_dt_r(0:dm),h_dt_l(0:dm)
      REAL curvature_t
      REAL gauss, gausst, edge, edget
      REAL dt(0:dm)
      REAL dt31(0:dm),dt41(0:dm)
      REAL d23(0:dm)
      REAL scratch31, scratch41, scratch01
      REAL dot_01, factor01, curvature_t1
      REAL h_forgot_l(0:dm)

      INTEGER iv_scra(0:max_num), m_scra(0:max_num)
      REAL v_scra(0:max_num)
      INTEGER idim
      REAL inv_norm_dt
      real ranmar

      sw_sacc = 0.0

      DO i_flip = 0, 3*n-1

        s0 = n * (1.0-ranmar())

        IF(s0.EQ.n)THEN
          s0=nm
        ENDIF

        ft(0) = field(0,s0) + trial * (ranmar()-.5)
        ft(1) = field(1,s0) + trial * (ranmar()-.5)
        ft(2) = field(2,s0) + trial * (ranmar()-.5)

        gauss = 0.0
        gausst = 0.0
        icount = 0

        nc = n_connectivity_s(s0)
        m_start = m_start_s(s0)

        do i=1,nc
          iv_scra(icount) = m_start
          s1 = s_neighbor_m(m_start)

          norm_dt = 0.0

          do idim=0,dm
             r_s1(idim) = field(idim,s1)
             dt(idim) = r_s1(idim)  - ft(idim)
             norm_dt  = norm_dt + dt(idim)*dt(idim)
          enddo

          v_scra(icount)  = norm_dt

          gauss  = gauss  + square_m(m_start)
          gausst = gausst + norm_dt

          m_start = m_next_m(m_start)
          icount = icount + 1
          if(icount .ge. max_num)then
             print *,'icount=',icount
             stop'too many links around'
          endif

        enddo  ! loop over the current sites

        old_action = gauss
        new_action = gausst

        IF(new_action .GE. old_action)THEN
          IF(exp(-(new_action-old_action)).lt.ranmar())goto22
        ENDIF

        sw_sacc = sw_sacc + 1.0

        field(0,s0) = ft(0)
        field(1,s0) = ft(1)
        field(2,s0) = ft(2)

        DO i = 0, icount - 1
          mem = iv_scra(i)
          m_start = m_back_m(mem)
          square_m(m_start)=v_scra(i)
          square_m(mem) = v_scra(i)
        ENDDO

   22   CONTINUE
      ENDDO
      sw_sacc = sw_sacc * inv_3n
      RETURN
  123 FORMAT(a)
      END

C************************************************************************C
C     Random number generator routines.                                  C
C************************************************************************C

      SUBROUTINE rmarin(ij,kl)
C This is the initialization routine for the random number generator
C     RANMAR()
C NOTE: The seed variables can have values between:    0 <= IJ <= 31328
C                                                      0 <= KL <= 30081
      REAL u(97), c, cd, cm
      INTEGER i97, j97, ivec
      LOGICAL test
      COMMON /raset1/ u, c, cd, cm, i97, j97, ivec, test

      IF( ij .LT. 0  .OR.  ij .GT. 31328  .OR.
     *    kl .LT. 0  .OR.  kl .GT. 30081 ) THEN
        PRINT '(A)', ' The first random number seed must have a value
     *between 0 and 31328'
        PRINT '(A)',' The second seed must have a value between 0 and
     *30081'
        STOP
      ENDIF
      i = mod(ij/177, 177) + 2
      j = mod(ij    , 177) + 2
      k = mod(kl/169, 178) + 1
      l = mod(kl,     169)
      DO 2 ii = 1, 97
        s = 0.0
        t = 0.5
        DO 3 jj = 1, 24
          m = mod(mod(i*j, 179)*k, 179)
          i = j
          j = k
          k = m
          l = mod(53*l+1, 169)
          IF (mod(l*m, 64) .ge. 32) then
            s = s + t
          ENDIF
          t = 0.5 * t
    3   CONTINUE
        u(ii) = s
    2 CONTINUE
      c = 362436.0 / 16777216.0
      cd = 7654321.0 / 16777216.0
      cm = 16777213.0 /16777216.0
      i97 = 97
      j97 = 33
      test = .true.
      RETURN
      END
C **************************************************************************
C **************************************************************************
      FUNCTION ranmar()
      IMPLICIT NONE
      REAL rand_u(97), rand_c, rand_cd, rand_cm, rand_uni, ranmar
      INTEGER rand_i97, rand_j97, rand_ivec
      LOGICAL rand_test
      COMMON /raset1/ rand_u, rand_c, rand_cd, rand_cm, rand_i97
     $     , rand_j97, rand_ivec, rand_test

      IF( .NOT. rand_test ) THEN
        PRINT '(A)',' Call the init routine (RMARIN) before calling RAN
     *MAR'
        STOP
      ENDIF
      rand_uni = rand_u(rand_i97) - rand_u(rand_j97)
      IF( rand_uni .LT. 0.0 ) rand_uni = rand_uni + 1.0
      rand_u(rand_i97) = rand_uni
      rand_i97 = rand_i97 - 1
      IF(rand_i97 .EQ. 0) rand_i97 = 97
      rand_j97 = rand_j97 - 1
      IF(rand_j97 .EQ. 0) rand_j97 = 97
      rand_c = rand_c - rand_cd
      IF( rand_c .LT. 0.0 ) rand_c = rand_c + rand_cm
      rand_uni = rand_uni - rand_c
      IF( rand_uni .LT. 0.0 ) rand_uni = rand_uni + 1.0
      ranmar = rand_uni
      RETURN
      END
C **************************************************************************

      BLOCK DATA
      REAL u(97), c, cd, cm
      INTEGER i97, j97, ivec
      LOGICAL test
      COMMON /raset1/ u, c, cd, cm, i97, j97, ivec, test
      DATA test /.false./
      END

